<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/core
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2023 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Core\Html\Elements;

use JchOptimize\ContainerFactory;
use JchOptimize\Core\Helper;
use JchOptimize\Core\Html\AttributesCollection;
use JchOptimize\Core\Html\HtmlElementInterface;
use JchOptimize\Core\Html\Processor;
use function array_merge;
use function strtolower;
/**
 * @method BaseElement id(string $value)
 * @method BaseElement class(string $value)
 * @method BaseElement hidden(string $value)
 * @method BaseElement style(string $value)
 * @method BaseElement title(string $value)
 * @method string|bool getId()
 * @method string|bool getClass()
 * @method string|bool getHidden()
 * @method string|bool getStyle()
 * @method string|bool getTitle()
 */
class BaseElement implements HtmlElementInterface
{
    protected AttributesCollection $attributes;
    protected string $name;
    protected bool $isXhtml;
    protected array $children = [];
    protected bool $omitClosingTag = \false;
    protected array $voidElements = ['area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr'];
    public function __construct()
    {
        $container = ContainerFactory::getContainer();
        $processor = $container->get(Processor::class);
        $this->isXhtml = Helper::isXhtml($processor->getHtml());
        $this->attributes = new AttributesCollection($this->isXhtml);
    }
    public function render() : string
    {
        $html = "<{$this->name}";
        $html .= $this->attributes->render();
        $html .= $this->isVoidElement($this->name) && $this->isXhtml ? ' />' : '>';
        if (!$this->isVoidElement($this->name) && !$this->omitClosingTag) {
            $html .= "{$this->renderChildren()}</{$this->name}>";
        }
        return $html;
    }
    public function attribute(string $name, $value = null, string $delimiter = null) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->attributes->setAttribute($name, $value, $delimiter);
        return $this;
    }
    public function remove($name) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->attributes->removeAttribute($name);
        return $this;
    }
    public function attributes(array $attributes) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->attributes->setAttributes($attributes);
        return $this;
    }
    public function __call($name, $arguments)
    {
        if (\substr($name, 0, 3) == 'get') {
            $name = strtolower(\substr($name, 3));
            return $this->attributeValue($name);
        }
        $value = $arguments[0] ?? null;
        $delimiter = $arguments[1] ?? null;
        return $this->attribute($name, $value, $delimiter);
    }
    public function data($name, $value = '') : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->attribute('data-' . $name, $value);
        return $this;
    }
    /**
     * @param HtmlElementInterface|string $child
     * @return $this
     */
    public function addChild($child) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->children[] = $child;
        return $this;
    }
    public function addChildren(array $children) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->children = array_merge($this->children, $children);
        return $this;
    }
    public function setName(string $name) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->name = $name;
        return $this;
    }
    public function attributeValue($name)
    {
        if ($this->attributes->has($name)) {
            return $this->attributes->getValue($name);
        }
        return \false;
    }
    public function hasAttribute(string $name) : bool
    {
        return $this->attributes->has($name);
    }
    public function firstOfAttributes(array $attributes)
    {
        foreach ($attributes as $name => $value) {
            if ($this->attributes->has($name)) {
                if ($this->attributes->isBoolean($name)) {
                    return $name;
                } elseif ($this->attributeValue($name) == $value) {
                    return $value;
                }
            }
        }
        return \false;
    }
    private function renderChildren() : string
    {
        $contents = '';
        foreach ($this->children as $child) {
            if ($child instanceof HtmlElementInterface) {
                $contents .= $child->render();
            } else {
                $contents .= $child;
            }
        }
        return $contents;
    }
    public function isVoidElement(string $name) : bool
    {
        return \in_array($name, $this->voidElements);
    }
    public function getChildren() : array
    {
        return $this->children;
    }
    /**
     * @param int $index
     * @param HtmlElementInterface|string $child
     * @return BaseElement
     */
    public function replaceChild(int $index, $child) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->children[$index] = $child;
        return $this;
    }
    public function hasChildren() : bool
    {
        return !empty($this->children);
    }
    public function __toString() : string
    {
        return $this->render();
    }
    public function setOmitClosingTag(bool $flag) : \JchOptimize\Core\Html\Elements\BaseElement
    {
        $this->omitClosingTag = $flag;
        return $this;
    }
}
